package ru.amse.cat.evlarchick.save;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import ru.amse.cat.evlarchick.textstructure.*;

import java.io.IOException;

public class SegmentsParserHandler extends DefaultHandler {
    public static final int SEGMENT = 0;
    public static final int FILEPATH = 1;
    public static final int DEADZONE = 2;

    private ITextStructure myTS;
    private ISegmentWithTranslation myCurrentSegmentWT;
    private int myMode;

    public SegmentsParserHandler(ITextStructure ts) {
        myTS = ts;
        myMode = -1;
    }

    public void characters (char ch[], int start, int length)
            throws SAXException {
        switch(myMode) {
            case SEGMENT:
                myCurrentSegmentWT.setTranslation(Coder.decode(new String(ch, start, length).trim()));
                break;
            case FILEPATH:
                try {
                    myTS.setText(new TextReader(new String(ch, start, length)).read());
                    //myTS.setFilePath(new String(ch, start, length));
                } catch (IOException e) {
                    throw new SAXException("Wrong filepath " + new String(ch, start, length), e);
                }
                break;
            case DEADZONE:
                break;
            default:
        }
        myMode = -1;
    }

    public void startElement (String uri, String localName,
                              String qName, Attributes attributes)
            throws SAXException {
        int start;
        int end;
        if ("segment".equals(qName)) {
            start = Integer.parseInt(attributes.getValue("start"));
            end = Integer.parseInt(attributes.getValue("end"));
            try {
                ISegmentWithTranslation segment = new SegmentWithTranslation(start, end, myTS);
                myTS.addSegmentWithTranslation(segment);
                myCurrentSegmentWT = segment;
                myMode = SEGMENT;
            } catch (SegmentIsTooBigException e) {
                e.printStackTrace();
            }
        } else if ("deadzone".equals(qName)) {
            start = Integer.parseInt(attributes.getValue("start"));
            end = Integer.parseInt(attributes.getValue("end"));
            try {
                ISegment segment = new Segment(start, end, myTS);
                myTS.addDeadZone(segment);
                myMode = DEADZONE;
            } catch (SegmentIsTooBigException e) {
                e.printStackTrace();
            }
        } else if ("filepath".equals(qName)) {
            myMode = FILEPATH;
        }
    }
}
