package ru.amse.cat.evlarchick.save;

import org.xml.sax.*;

import javax.xml.parsers.*;
import java.io.*;
import ru.amse.cat.evlarchick.textstructure.*;

public class TextStructureSaver implements ITextStructureSaver {
    private boolean myWasCatFileOpened;

    public ITextStructure load(File f) throws SavingException {
        String filepath = f.getAbsolutePath();
        if (filepath == null) {
            return null;
        } else if (!filepath.endsWith(".cat")) {
            myWasCatFileOpened = false;
            String originalText = null;
            try {
                originalText = new TextReader(filepath).read();
            } catch (IOException e) {
                throw new SavingException(e);
            }

            return new TextStructure(originalText, filepath);

        }

        myWasCatFileOpened = true;
        try {
            SAXParser sp = SAXParserFactory.newInstance().newSAXParser();
            ITextStructure ts = new TextStructure(null);
            sp.parse(f, new SegmentsParserHandler(ts));
            return ts;
        } catch (SAXException e) {
            //e.printStackTrace();
            throw new SavingException(e);
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
            throw new SavingException(e);
        } catch (IOException e) {
            throw new SavingException(e);
        }
    }

    public void save(ITextStructure ts, File file) throws SavingException {
        FileWriter fw = null;
        try {
            String textPath;
            String filepath = file.getAbsolutePath();
            if (!filepath.endsWith(".cat")) {
                file = new File(filepath + ".cat");
                filepath = file.getAbsolutePath();
            }
            textPath = filepath.substring(0, filepath.length() - 4) + ".txt";
            fw = new FileWriter(textPath);
            fw.write(ts.getOriginalText());
            fw.close();

            fw = new FileWriter(file);
            fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<file>\n");
            fw.write("    <filepath>" + textPath + "</filepath>\n");
            for (ISegmentWithTranslation s : ts.getSegments()) {
                fw.write("    <segment start=\"" + s.getStart() + "\" end=\"" + s.getEnd() + "\">\n        ");
                fw.write(Coder.code(s.getTranslation()));
                fw.write("\n    </segment>\n");
            }
            for (ISegment s : ts.getDeadZones()) {
                fw.write("    <deadzone start=\"" + s.getStart() + "\" end=\"" + s.getEnd() + "\">");
                fw.write("\n    </deadzone>\n");
            }
            fw.write("</file>");
        } catch(IOException e) {
            throw new SavingException(e);
        } finally {
            if (fw != null) {
                try {
                    fw.close();
                } catch (IOException e) {}
            }
            myWasCatFileOpened = true;
        }
    }

    public boolean wasCATFileLoaded() {
        return myWasCatFileOpened;
    }
}
