package ru.amse.cat.evlarchick.textstructure;

import java.io.Reader;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

public class TextInfo implements ITextInfo {
    private Reader myReader;
    protected List<ISegment> mySegments;
    protected List<ISegment> myDeadZones;

    public TextInfo(Reader r) {
        this(r, new ArrayList<ISegment>(), new ArrayList<ISegment>());
    }

    public TextInfo(Reader r, List<? extends ISegment> segments, List<ISegment> deadzones) {
        myReader = r;
        mySegments = new ArrayList<ISegment>(segments);
        myDeadZones = new ArrayList<ISegment>(deadzones);
    }


    public Reader getReader() {
        return myReader;
    }

    public List<ISegment> getSegments() {
        return mySegments;
    }

    public void addSegment(ISegment s) {
        for (int i = 0; i < mySegments.size(); i++) {
            if (mySegments.get(i).getStart() >= s.getEnd()) {
                mySegments.add(i, s);
                return;
            }
        }
        mySegments.add(s);
    }

    public List<ISegment> getDeadZones() {
        return myDeadZones;
    }

    public void addDeadZone(ISegment segment) {
        for (int i = 0; i < myDeadZones.size(); i++) {
            if (myDeadZones.get(i).getStart() >= segment.getEnd()) {
                myDeadZones.add(i, segment);
                return;
            }
        }
        myDeadZones.add(segment);
    }

    public int getSegmentIndexByTextPosition(int index) {
        int idx = 0;
        for (ISegment s : mySegments) {
            if (s.getStart() <= index && s.getEnd() > index) {
                return idx;
            }
            idx++;
        }
        return -1;
    }

    public int getDeadZoneIndexByTextPosition(int index) {
        int idx = 0;
        for (ISegment s : myDeadZones) {
            if (s.getStart() <= index && s.getEnd() > index) {
                return idx;
            }
            idx++;
        }
        return -1;
    }

    public void clear() {
        mySegments.clear();
        myDeadZones.clear();
    }

    public List<ISegment> getCommonSegmentsList() {
        Iterator<ISegment> it1 = myDeadZones.iterator();
        Iterator<ISegment> it2 = mySegments.iterator();

        List<ISegment> commonList = new LinkedList<ISegment>();

        ISegment deadZone = (it1.hasNext()) ? it1.next() : null;
        ISegment segment = (it2.hasNext()) ? it2.next() : null;

        if (!((deadZone == null) && (segment == null))) {
            do {
                if (segment == null)  {
                    commonList.add(deadZone);
                    deadZone = (it1.hasNext()) ? it1.next() : null;
                    if (deadZone == null) {
                        break;
                    }
                } else if(deadZone == null) {
                    commonList.add(segment);
                    segment = (it2.hasNext()) ? it2.next() : null;
                    if (segment == null) {
                        break;
                    }
                } else {
                    if (deadZone.getStart() < segment.getStart()) {
                        commonList.add(deadZone);
                        deadZone = (it1.hasNext()) ? it1.next() : null;
                    } else {
                        commonList.add(segment);
                        segment = (it2.hasNext()) ? it2.next() : null;
                    }
                }
            } while (true);
        }
        return commonList;
    }
}
