package ru.amse.cat.evlarchick.ui;

import ru.amse.cat.evlarchick.textstructure.*;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

public class Finder implements IFinder {
    private ITextStructure myTextStructure;
    private ITextInfo myTextInfo;
    private ITextInfo myTranslationInfo;
    private List<ISegment> myLeftList;
    private List<ISegment> myRightList;

    public void setInitialTextInfo(ITextInfo info) {
        myTextInfo = info;
    }

    public void setTanslationInfo(ITextInfo info) {
        myTranslationInfo = info;
    }

    public void setTextStructure(ITextStructure ts) {
        myTextStructure = ts;
    }

    public void processFind(String text, boolean isDZ) {
        LinkedList<FoundElementInfo> results = find(text, myTextStructure.getOriginalText());

        for (FoundElementInfo info : results) {
            addSegment(info.getMyStart(), info.getEnd(), info.getTranslationDiff(), isDZ, myTextStructure.getOriginalText());
        }
    }

    public void processReplace(String text, String replaceWith) {
        LinkedList<FoundElementInfo> results = find(text, myTextStructure.getOriginalText());

        if ((results == null) || results.size() == 0) {
            return;
        }
        int diff = text.length() - replaceWith.length();

        if (diff == 0) {
            for (FoundElementInfo info : results) {
                addReplacedSegment(info.getMyStart(), info.getEnd(), info.getTranslationDiff(), replaceWith);
            }
        } else if (diff < 0) {
            replaceReverse(results, -diff, replaceWith);
        } else {
            replace(results, -diff, replaceWith);
        }
    }

    private void replace(LinkedList<FoundElementInfo> results, int diff, String replaceWith) {
        FoundElementInfo res = results.getFirst();
        int count = 0;
        boolean wereAllResultsProcessed = false;

        class ReplaceInfo {
            private int myStart;
            private int myEnd;
            private int myTranslationDiff;
            private String myTranslation;

            ReplaceInfo(int start, int end, int translationDiff, String translation) {
                myStart = start;
                myEnd = end;
                myTranslationDiff = translationDiff;
                myTranslation = translation;
            }

            public int getStart() {
                return myStart;
            }

            public int getEnd() {
                return myEnd;
            }

            public int getTranslationDiff() {
                return myTranslationDiff;
            }

            public String getTranslation() {
                return myTranslation;
            }
        }
        LinkedList<ReplaceInfo> segmentsToAdd = new LinkedList<ReplaceInfo>();
        for (Iterator<ISegment> it = myTranslationInfo.getCommonSegmentsList().iterator(); it.hasNext(); ) {
            ISegment s = it.next();

            int start = s.getStart();
            int end = s.getEnd();

            while((!wereAllResultsProcessed) && (start > res.getMyStart() + res.getTranslationDiff())) {
                segmentsToAdd.add(new ReplaceInfo(res.getMyStart(), res.getEnd(), res.getTranslationDiff() + count * diff, replaceWith));
                results.removeFirst();
                count++;
                if (results.size() != 0) {
                    res = results.getFirst();
                } else {
                    wereAllResultsProcessed = true;
                }
            }

            s.setStart(start + count * diff);
            s.setEnd(end + count * diff);
        }

        while (results.size() > 0) {
            res = results.getFirst();
            segmentsToAdd.add(new ReplaceInfo(res.getMyStart(), res.getEnd(), res.getTranslationDiff() + count * diff, replaceWith));
            results.removeFirst();
            count++;
        }

        for (ReplaceInfo info : segmentsToAdd) {
            addReplacedSegment(info.getStart(), info.getEnd(), info.getTranslationDiff(), info.getTranslation());
        }
    }

    private void replaceReverse(LinkedList<FoundElementInfo> results, int diff, String replaceWith) {
        FoundElementInfo res = results.getLast();
        int count = results.size() - 1;

        List<ISegment> commonList = myTranslationInfo.getCommonSegmentsList();
        forloop:
        for (ListIterator<ISegment> it = commonList.listIterator(commonList.size()); it.hasPrevious(); ) {
            ISegment s = it.previous();
            int start = s.getStart();
            int end = s.getEnd();
            while (start < res.getEnd() + res.getTranslationDiff()) {
                addReplacedSegment(res.getMyStart(), res.getEnd(), res.getTranslationDiff() + count * diff, replaceWith);
                results.removeLast();
                if (count == 0) {
                    break forloop;
                } else {
                    count--;
                    res = results.getLast();
                }
            }

            if (start + (count + 1) * diff >= res.getEnd() + res.getTranslationDiff()) {
                s.setStart(start + (count + 1) * diff);
                s.setEnd(end + (count + 1) * diff);
            }
        }

        while (results.size() > 0) {
            res = results.getLast();
            addReplacedSegment(res.getMyStart(), res.getEnd(), res.getTranslationDiff() + count * diff, replaceWith);
            results.removeLast();
            count--;
        }
    }

    private void addReplacedSegment(int start, int end, int translationDiff, String translation) {
        try {
            ISegmentWithTranslation newSegment = new SegmentWithTranslation(translation,
                    start, end, myTextStructure);
            myTextStructure.addSegmentWithTranslation(newSegment);
            myTextInfo.addSegment(newSegment);
            ISegment s = new SimpleSegment(start + translationDiff, start + translationDiff + translation.length());
            myTranslationInfo.addSegment(s);
        } catch(SegmentIsTooBigException e) {
            System.out.println(e.getMessage());
        }
    }

    private void addSegment(int start, int end, int translationDiff, boolean isDZ, String sb) {
        if (isDZ) {
            try {
                ISegment newSegment = new Segment(start, end, myTextStructure);
                myTextStructure.addDeadZone(newSegment);
                myTextInfo.addDeadZone(newSegment);
                ISegment s = new SimpleSegment(start + translationDiff, end + translationDiff);
                myTranslationInfo.addDeadZone(s);
            } catch(SegmentIsTooBigException e) {
                System.out.println(e.getMessage());
            }
        } else {
            try {
                ISegmentWithTranslation newSegment = new SegmentWithTranslation(sb.substring(start, end),
                        start, end, myTextStructure);
                myTextStructure.addSegmentWithTranslation(newSegment);
                myTextInfo.addSegment(newSegment);
                ISegment s = new SimpleSegment(start + translationDiff, end + translationDiff);
                myTranslationInfo.addSegment(s);
            } catch(SegmentIsTooBigException e) {
                System.out.println(e.getMessage());
            }
        }
    }

    private LinkedList<FoundElementInfo> find(String text, String fromText) {
        myLeftList = myTextStructure.getCommonSegmentList();
        myRightList = myTranslationInfo.getCommonSegmentsList();

        List<ISegment> current = myLeftList;
        int size = current.size();
        int i = 0;
        ISegment s = (i < size) ? current.get(i) : null;
        LinkedList<FoundElementInfo> results = new LinkedList<FoundElementInfo>();
        int translationDiff;
        int index = -1;

        Pattern p = Pattern.compile(text);
        Matcher m = p.matcher(fromText);

        indexLoop:
        while (m.find(index + 1)) {
            index = m.start();
            if (s != null) {
                if (s.getStart() > index) {
                    translationDiff = (i > 0) ? myRightList.get(i - 1).getEnd() -
                            myLeftList.get(i - 1).getEnd() : 0;
                    results.add(new FoundElementInfo (index, index + text.length(),
                            translationDiff));
                    continue;
                }
                while ((s != null) && (s.getStart() <= index)){
                    if (s.getEnd() > index) {
                        index = s.getEnd();
                        i++;
                        s = (i < size) ? current.get(i) : null;
                        continue indexLoop;
                    }

                    i++;
                    s = (i < size) ? current.get(i) : null;
                }
                translationDiff = (i > 0) ? myRightList.get(i - 1).getEnd() -
                        myLeftList.get(i - 1).getEnd() : 0;
                results.add(new FoundElementInfo(index, index + text.length(), translationDiff));
            } else {
                translationDiff = (i > 0) ? myRightList.get(i - 1).getEnd() -
                        myLeftList.get(i - 1).getEnd() : 0;
                results.add(new FoundElementInfo (index, index + text.length(), translationDiff));
            }
        }

        return results;
    }

    private static class FoundElementInfo {
        private int myStart;
        private int myEnd;
        private int myTranslationDiff;

        public FoundElementInfo(int myStart, int myEnd, int myTranslationDiff) {
            this.myStart = myStart;
            this.myEnd = myEnd;
            this.myTranslationDiff = myTranslationDiff;
        }

        public int getMyStart() {
            return myStart;
        }

        public int getEnd() {
            return myEnd;
        }

        public int getTranslationDiff() {
            return myTranslationDiff;
        }
    }
}
